#include "fds.h"
#include "fstorage.h"

static volatile uint8_t write_flag=0;

static void test_fds_evt_handler(fds_evt_t const * const p_fds_evt)
{
    switch (p_fds_evt->id)
    {
        case FDS_EVT_INIT:
            if (p_fds_evt->result != FDS_SUCCESS)
            {
                // Initialization failed.
                NRF_LOG_PRINTF("Initialization failed\n" );
            }
            break;
				case FDS_EVT_WRITE:
						if (p_fds_evt->result == FDS_SUCCESS)
						{
                write_flag = 1;
                NRF_LOG_PRINTF("set write_flag\n" );
						}
						break;
        default:
            break;
    }
}

static ret_code_t test_fds_write_rec1(void)
{
    #define FILE_ID     0x1111
    #define REC_KEY1    0x1111
__ALIGN(sizeof(uint32_t))
    uint8_t device_data[] = "Tablet1";
    fds_record_t        record;
    fds_record_desc_t   record_desc;
    fds_record_chunk_t  record_chunk;
    // Set up data.
    record_chunk.p_data         = device_data;
    record_chunk.length_words   = (sizeof(device_data)+1)/4;
    // Set up record.
    record.file_id              = FILE_ID;
    record.key              		= REC_KEY1;
    record.data.p_chunks       = &record_chunk;
    record.data.num_chunks   = 1;

    ret_code_t ret = fds_record_write(&record_desc, &record);
    if (ret != FDS_SUCCESS)
    {
        return ret;
    }
    NRF_LOG_PRINTF("Writing Record ID = %d \r\n",record_desc.record_id);
    return NRF_SUCCESS;
}

static ret_code_t test_fds_write_rec2(void)
{
    #define FILE_ID     0x1111
    #define REC_KEY2    0x2222
__ALIGN(sizeof(uint32_t))
    uint8_t device_data[] = "data: abcdef";
    fds_record_t        record;
    fds_record_desc_t   record_desc;
    fds_record_chunk_t  record_chunk;
    // Set up data.
    record_chunk.p_data         = device_data;
    record_chunk.length_words   = (sizeof(device_data)+1)/4;
    // Set up record.
    record.file_id              = FILE_ID;
    record.key              		= REC_KEY2;
    record.data.p_chunks       = &record_chunk;
    record.data.num_chunks   = 1;

    ret_code_t ret = fds_record_write(&record_desc, &record);
    if (ret != FDS_SUCCESS)
    {
        return ret;
    }
    NRF_LOG_PRINTF("Writing Record ID = %d \r\n",record_desc.record_id);
    return NRF_SUCCESS;
}

static ret_code_t test_fds_write_rec3(void)
{
    #define FILE_ID     0x1111
    #define REC_KEY3    0x3333
__ALIGN(sizeof(uint32_t))
    uint32_t device_data = 26;
    fds_record_t        record;
    fds_record_desc_t   record_desc;
    fds_record_chunk_t  record_chunk;
    // Set up data.
    record_chunk.p_data         = &device_data;
    record_chunk.length_words   = 1;
    // Set up record.
    record.file_id              = FILE_ID;
    record.key              		= REC_KEY3;
    record.data.p_chunks       = &record_chunk;
    record.data.num_chunks   = 1;

    ret_code_t ret = fds_record_write(&record_desc, &record);
    if (ret != FDS_SUCCESS)
    {
        return ret;
    }
    NRF_LOG_PRINTF("Writing Record ID = %d \r\n",record_desc.record_id);
    return NRF_SUCCESS;
}

static ret_code_t test_fds_read(void)
{
		#define FILE_ID     0x1111
		fds_flash_record_t  flash_record;
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok ={0};//Important, make sure you zero init the ftok token
		uint32_t *data;
		uint32_t err_code;
		
		NRF_LOG_PRINTF("Start searching... \r\n");
		// Loop until all records with the given key and file ID have been found.
		while (fds_record_find(FILE_ID, 0x1111, &record_desc, &ftok) == FDS_SUCCESS)
		{
				err_code = fds_record_open(&record_desc, &flash_record);
				if ( err_code != FDS_SUCCESS)
				{
					return err_code;		
				}
				
				NRF_LOG_PRINTF("Found Record ID = %d\r\n",record_desc.record_id);
				NRF_LOG_PRINTF("Data = ");
				data = (uint32_t *) flash_record.p_data;
				for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
				{
					NRF_LOG_PRINTF("0x%8x ",data[i]);
				}
				NRF_LOG_PRINTF("\r\n");
     }
    ftok.p_addr = 0;
    ftok.page = 0;
		while (fds_record_find(FILE_ID, 0x2222, &record_desc, &ftok) == FDS_SUCCESS)
		{
				err_code = fds_record_open(&record_desc, &flash_record);
				if ( err_code != FDS_SUCCESS)
				{
					return err_code;		
				}
				
				NRF_LOG_PRINTF("Found Record ID = %d\r\n",record_desc.record_id);
				NRF_LOG_PRINTF("Data = ");
				data = (uint32_t *) flash_record.p_data;
				for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
				{
					NRF_LOG_PRINTF("0x%8x ",data[i]);
				}
				NRF_LOG_PRINTF("\r\n");
     }
    ftok.p_addr = 0;
    ftok.page = 0;
		while (fds_record_find(FILE_ID, 0x3333, &record_desc, &ftok) == FDS_SUCCESS)
		{
				err_code = fds_record_open(&record_desc, &flash_record);
				if ( err_code != FDS_SUCCESS)
				{
					return err_code;		
				}
				
				NRF_LOG_PRINTF("Found Record ID = %d\r\n",record_desc.record_id);
				NRF_LOG_PRINTF("Data = ");
				data = (uint32_t *) flash_record.p_data;
				for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
				{
					NRF_LOG_PRINTF("0x%8x ",data[i]);
				}
				NRF_LOG_PRINTF("\r\n");
    }      
				// Access the record through the flash_record structure.
				// Close the record when done.
				err_code = fds_record_close(&record_desc);
				if (err_code != FDS_SUCCESS)
				{
					return err_code;	
				}
//		}
		return NRF_SUCCESS;
}

static ret_code_t test_fds_init (void)
{
    ret_code_t ret = fds_register(test_fds_evt_handler);
    if (ret != FDS_SUCCESS)
    {
        return ret;
    }
    ret = fds_init();
    if (ret != FDS_SUCCESS)
    {
        return ret;
    }
    return NRF_SUCCESS;
}

void fds_test(void)
{
    uint32_t err_code;
    err_code = test_fds_init();
    APP_ERROR_CHECK(err_code);
    write_flag = 0;
    err_code = test_fds_write_rec1();
    APP_ERROR_CHECK(err_code);
    while (write_flag==0);
    write_flag = 0;
    err_code = test_fds_write_rec2();
    APP_ERROR_CHECK(err_code);
    while (write_flag==0);
    write_flag = 0;
    err_code = test_fds_write_rec3();
    APP_ERROR_CHECK(err_code);
    while (write_flag==0);
    test_fds_read();
}
